@@ -186,10 +186,10 @@ static char * mypc_xpm[] = {
 "                  "};
 
 
-
 static void gtk_dir_tree_class_init          (GtkDirTreeClass *klass);
 static void gtk_dir_tree_init                (GtkDirTree      *dir_tree);
 
+
 static GtkCTreeClass *parent_class = NULL;
 
 
@@ -232,10 +232,12 @@ static void
 gtk_dir_tree_class_init (GtkDirTreeClass *klass)
 {
   GtkWidgetClass *widget_class;
+  GtkObjectClass *object_class;
   
-  widget_class = (GtkWidgetClass*) klass;
   parent_class = gtk_type_class (gtk_ctree_get_type ());
- 
+  widget_class = (GtkWidgetClass*) klass;
+  object_class = (GtkObjectClass *) klass;
+
 }
 
 static void
@@ -267,8 +269,9 @@ gtk_dir_tree_init (GtkDirTree *dir_tree)
 #endif
 
   dir_tree->local_hostname = g_strdup(localhost);
+  g_object_set(G_OBJECT(widget), "n_columns", 1, "tree_column", 0, NULL);
+
 
-  gtk_ctree_construct(GTK_CTREE(dir_tree), 1, 0, NULL);
   gtk_clist_set_row_height (GTK_CLIST (dir_tree), 18);
 
   dir_tree->my_pc=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap,
@@ -352,10 +355,14 @@ static gboolean check_for_subdir(gchar *path, gboolean show_hidden)
   {
     while((dirent=readdir(dir))!=NULL)
     {
-      if(accept_dirname(dirent->d_name,show_hidden))
+	if(accept_dirname(dirent->d_name,show_hidden))
       {
-        npath=g_strconcat(path,G_DIR_SEPARATOR_S,dirent->d_name,NULL);
-
+#ifndef G_OS_WIN32
+        npath=g_strconcat(path,dirent->d_name,G_DIR_SEPARATOR_S,NULL);
+#else
+        /* the M$ stat is somewhat pedantic ... */
+        npath=g_strconcat(path,dirent->d_name,NULL);
+#endif
         if(0!=stat(npath,&statbuf))
         {
           g_free(npath);
@@ -428,10 +435,7 @@ expand_tree(GtkCTree *ctree,GtkCTreeNode *parent_node, gpointer data)
       }
       while((dirent=readdir(dir))!=NULL)
       {
-        if(strlen(parent_dirnode->path) == 1)
-          path=g_strconcat(G_DIR_SEPARATOR_S,dirent->d_name,NULL);
-        else
-          path=g_strconcat(parent_dirnode->path,G_DIR_SEPARATOR_S,dirent->d_name,NULL);
+        path=g_strconcat(parent_dirnode->path,dirent->d_name,NULL);
         if (stat_subdirs)
         {
           if(0!=stat(path,&statbuf))
@@ -445,7 +449,7 @@ expand_tree(GtkCTree *ctree,GtkCTreeNode *parent_node, gpointer data)
         {
           DIR *dir_sub;
           dirnode=g_malloc0(sizeof(GtkDirTreeNode));
-          dirnode->path=g_strconcat(path,NULL);
+          dirnode->path=g_strconcat(path,G_DIR_SEPARATOR_S,NULL);
           text=dirent->d_name;
           if (stat_subdirs)
           {
@@ -529,7 +533,7 @@ gtk_dir_tree_open_dir(GtkDirTree *dir_tree, const gchar *path)
            g_free(aux_path);
            aux_path = g_strdup(root);
         } else {
-           aux_path[length - 2] = '\0';
+           aux_path[length - 1] = '\0';
         }
      } else if(strcmp(aux_path + length - 3, root2) == 0){
         if(length == 3) {
@@ -539,7 +543,7 @@ gtk_dir_tree_open_dir(GtkDirTree *dir_tree, const gchar *path)
            gint i = length - 4;
            while(i >= 0){
               if(aux_path[i] == root[0]){
-                   aux_path[i] = '\0';
+                   aux_path[i+1] = '\0';
                    break;
               }
               i--;
@@ -553,7 +557,7 @@ gtk_dir_tree_open_dir(GtkDirTree *dir_tree, const gchar *path)
            gint i = length - 5;
            while(i >= 0){
               if(aux_path[i] == root[0]){
-                   aux_path[i] = '\0';
+                   aux_path[i+1] = '\0';
                    break;
               }
               i--;
@@ -564,7 +568,7 @@ gtk_dir_tree_open_dir(GtkDirTree *dir_tree, const gchar *path)
            g_free(aux_path);
            aux_path = g_strdup(root);
         } else {
-           aux_path[length - 3] = '\0';
+           aux_path[length - 2] = '\0';
         }
      }
   }
