@@ -489,7 +489,20 @@ select_icon(GtkIconList *iconlist,
   filesel = GTK_ICON_FILESEL(data);
 
   if(item->type != GTK_FILE_LIST_FOLDER){
-    gtk_entry_set_text(GTK_ENTRY(filesel->file_entry), icon->label);
+    GList *list = iconlist->selection;
+    if(iconlist->selection_mode == GTK_SELECTION_MULTIPLE && list != NULL){
+      gchar *text = g_strdup(((GtkIconListItem *)list->data)->label); 
+      list = list->next;
+      while(list){
+        text = g_strconcat(text,";",((GtkIconListItem *)list->data)->label,NULL);
+        list = list->next;
+      }
+      text = g_strconcat(text,";",icon->label,NULL);
+      gtk_entry_set_text(GTK_ENTRY(filesel->file_entry), text);
+      g_free(text);
+    } else {
+      gtk_entry_set_text(GTK_ENTRY(filesel->file_entry), icon->label);
+    }
     return TRUE;
   }else{
     gtk_entry_set_text(GTK_ENTRY(filesel->file_entry), "");
@@ -598,7 +611,10 @@ real_set_file(GtkWidget *widget, gpointer data)
       item = (GtkIconListItem *)list->data;
       text = ((GtkFileListItem *)item->link)->file_name;
       if(strcmp(text, file) == 0){
+         item->state = GTK_STATE_SELECTED;
+/*
          gtk_icon_list_select_icon(GTK_ICON_LIST(filesel->file_list), item);
+*/
          break;
       }
       list = list->next;
