@@ -61,6 +61,7 @@ static guint signals[LAST_SIGNAL] = {0};
 static void gtk_icon_list_class_init 		(GtkIconListClass *class);
 static void gtk_icon_list_init 			(GtkIconList *icon_list);
 static void gtk_icon_list_destroy 		(GtkObject *object);
+static void gtk_icon_list_finalize 		(GObject *object);
 
 static void gtk_icon_list_size_allocate		(GtkWidget *widget,
                                                  GtkAllocation *allocation);
@@ -195,14 +196,17 @@ static void
 gtk_icon_list_class_init (GtkIconListClass *klass)
 {
   GtkObjectClass *object_class;
+  GObjectClass *gobject_class;
   GtkWidgetClass *widget_class;
 
   parent_class = gtk_type_class (GTK_TYPE_FIXED);
 
   object_class = (GtkObjectClass *) klass;
+  gobject_class = (GObjectClass *) klass;
   widget_class = (GtkWidgetClass *) klass;
 
   object_class->destroy = gtk_icon_list_destroy;
+  gobject_class->finalize = gtk_icon_list_finalize;
 
   widget_class->realize = gtk_icon_list_realize;
 
@@ -304,12 +308,17 @@ reorder_icons(GtkIconList *iconlist)
   widget = GTK_WIDGET(iconlist);
 
   if(iconlist->freeze_count > 0) return;
+
 /*
   gdk_threads_enter();
 */
 
   old_width = widget->allocation.width;
   old_height = widget->allocation.height;
+  if(GTK_WIDGET_REALIZED(widget)){
+    if(GTK_IS_VIEWPORT(widget->parent))
+     gdk_window_get_size(GTK_VIEWPORT(widget->parent)->view_window, &old_width, &old_height);
+  }
 
   y = iconlist->row_spacing;
   x = iconlist->col_spacing;
@@ -371,6 +380,10 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
   icon->x = x;
   icon->y = y;
 
+/*
+  if(x == old_x && y == old_y) return;
+*/
+
   item_size_request(iconlist, icon, &req);
   req1 = icon->pixmap->requisition;
   req2 = icon->entry->requisition;
@@ -438,10 +451,16 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
 static void
 gtk_icon_list_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
-  GtkAllocation old = widget->allocation;
+  GtkAllocation *old = gtk_object_get_data(GTK_OBJECT(widget),"viewport");
   GTK_WIDGET_CLASS(parent_class)->size_allocate(widget, allocation);
-  if(old.width != allocation->width || old.height != allocation->height)
-    reorder_icons(GTK_ICON_LIST(widget));
+  if(GTK_WIDGET_REALIZED(widget) && old){
+    gint new_width, new_height;
+    gdk_window_get_size(GTK_VIEWPORT(widget->parent)->view_window, &new_width, &new_height);
+    if(old->width != new_width || old->height != new_height)
+      reorder_icons(GTK_ICON_LIST(widget));
+    old->width = new_width; 
+    old->height = new_height; 
+  }
 }
 
 
@@ -483,7 +502,13 @@ gtk_icon_list_realize(GtkWidget *widget)
     icons = icons->next;
   }
 
+/*  
+  if(GTK_IS_VIEWPORT(widget->parent) && GTK_WIDGET_REALIZED(widget->parent)){
+    GtkAllocation *allocation = gtk_object_get_data(GTK_OBJECT(widget),"viewport");
+    gdk_window_get_size(GTK_VIEWPORT(widget->parent)->view_window, &allocation->width, &allocation->height);
+  }
   reorder_icons(iconlist);
+*/
 }
 
 
@@ -649,7 +674,9 @@ deactivate_entry(GtkIconList *iconlist)
        gdk_gc_unref(gc);
      }
 
+/*
      iconlist->active_icon->state = GTK_STATE_NORMAL;
+*/
      iconlist->active_icon = NULL;
   }
 
@@ -672,6 +699,12 @@ select_icon(GtkIconList *iconlist, GtkIconListItem *item, GdkEvent *event)
   if(iconlist->mode != GTK_ICON_LIST_ICON){ 
     if(!deactivate_entry(iconlist)) return;
 
+    if(item->state != GTK_STATE_NORMAL && iconlist->selection_mode == GTK_SELECTION_MULTIPLE) {
+       unselect_icon(iconlist, item, event);
+       return;
+    }
+    if(item->state == GTK_STATE_SELECTED) return;
+
     if(item->entry && GTK_WIDGET_REALIZED(item->entry)){
       GtkStyle *style = gtk_style_copy(item->entry->style);
       style->bg[GTK_STATE_ACTIVE] = style->base[GTK_STATE_SELECTED];
@@ -698,9 +731,20 @@ select_icon(GtkIconList *iconlist, GtkIconListItem *item, GdkEvent *event)
       }
     }
   }
-  if(item->state == GTK_STATE_SELECTED) return;
   iconlist->selection = g_list_append(iconlist->selection, item);
 
+/*
+{
+  GList *aux = iconlist->selection;
+  while(aux){
+  GtkIconListItem *kk = (GtkIconListItem *)aux->data;
+  printf("SELECTION %s\n",kk->entry_label);
+  aux = aux->next;
+  }
+}
+*/
+
+
   item->state = GTK_STATE_SELECTED;  
   if(item->entry) gtk_widget_grab_focus(item->entry);
 }
@@ -725,7 +769,7 @@ unselect_icon(GtkIconList *iconlist, GtkIconListItem *item, GdkEvent *event)
   if(selection){
        iconlist->selection = g_list_remove_link(iconlist->selection, selection);
   }
- 
+
   item->state = GTK_STATE_NORMAL;
 
   if(iconlist->mode != GTK_ICON_LIST_ICON){
@@ -771,10 +815,13 @@ GtkWidget*
 gtk_icon_list_new (guint icon_width, GtkIconListMode mode)
 {
   GtkIconList *icon_list;
+  GtkAllocation *allocation;
 
   icon_list = gtk_type_new (gtk_icon_list_get_type ());
 
   gtk_icon_list_construct(icon_list, icon_width, mode);
+  allocation = g_new0(GtkAllocation, 1);
+  gtk_object_set_data(GTK_OBJECT(icon_list), "viewport", allocation);
 
   return GTK_WIDGET (icon_list);
 }
@@ -866,6 +913,21 @@ gtk_icon_list_destroy (GtkObject *object)
     (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
+static void
+gtk_icon_list_finalize (GObject *object)
+{
+  GtkIconList *icon_list;
+  GtkAllocation *allocation;
+
+  icon_list = GTK_ICON_LIST (object);
+
+  allocation = gtk_object_get_data(GTK_OBJECT(icon_list), "viewport");
+  if(allocation) g_free(allocation);
+  gtk_object_set_data(GTK_OBJECT(icon_list), "viewport", NULL);
+
+  if (G_OBJECT_CLASS (parent_class)->finalize)
+    (*G_OBJECT_CLASS (parent_class)->finalize) (object);
+}
 
 void
 gtk_icon_list_set_background (GtkIconList *iconlist, GdkColor *color)
@@ -1573,10 +1635,19 @@ gtk_icon_list_set_pixmap(GtkIconListItem *item,
 			 GdkPixmap *pixmap, 
 			 GdkBitmap *mask)
 {
+  gint x, y;
+  GtkWidget *widget = gtk_widget_get_parent(item->pixmap);
+
+  x = item->x;
+  y = item->y;
+  
+  gtk_container_remove(GTK_CONTAINER(widget), item->pixmap);
+  x = item->pixmap->allocation.x;
+  y = item->pixmap->allocation.y;
 
-  if(item->pixmap) gtk_widget_destroy(item->pixmap);  
   item->pixmap = gtk_pixmap_new(pixmap, mask);
-
+  gtk_widget_show(item->pixmap);
+  gtk_fixed_put(GTK_FIXED(widget), item->pixmap, x, y);
 }
 
 void 
