@@ -28,15 +28,18 @@
 #include "gtkplotbox.h"
 #include "gtkpsfont.h"
 
+#define P_(string) string
+
 static void gtk_plot_box_class_init 	(GtkPlotBoxClass *klass);
 static void gtk_plot_box_init 		(GtkPlotBox *data);
-static void gtk_plot_box_set_arg        (GtkObject *object,
-                                         GtkArg    *arg,
-                                         guint      arg_id);
-static void gtk_plot_box_get_arg        (GtkObject *object,
-                                         GtkArg    *arg,
-                                         guint      arg_id);
-
+static void gtk_plot_box_set_property   (GObject *object,
+                                         guint prop_id,
+                                         const GValue *value,
+                                         GParamSpec *pspec);
+static void gtk_plot_box_get_property   (GObject *object,
+                                         guint prop_id,
+                                         GValue *value,
+                                         GParamSpec *pspec);
 static void gtk_plot_box_draw_legend	(GtkPlotData *data, 
 					 gint x, gint y);
 static void gtk_plot_box_draw_symbol	(GtkPlotData *data,
@@ -89,6 +92,7 @@ gtk_plot_box_class_init (GtkPlotBoxClass *klass)
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkPlotDataClass *data_class;
+  GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
   parent_class = gtk_type_class (gtk_plot_data_get_type ());
 
@@ -96,13 +100,16 @@ gtk_plot_box_class_init (GtkPlotBoxClass *klass)
   widget_class = (GtkWidgetClass *) klass;
   data_class = (GtkPlotDataClass *) klass;
 
-  object_class->set_arg = gtk_plot_box_set_arg;
-  object_class->get_arg = gtk_plot_box_get_arg;
+  gobject_class->set_property = gtk_plot_box_set_property;
+  gobject_class->get_property = gtk_plot_box_get_property;
 
-  gtk_object_add_arg_type ("GtkPlotBox::orientation",
-                           GTK_TYPE_UINT,
-                           GTK_ARG_READWRITE,
-                           ARG_ORIENTATION);
+  g_object_class_install_property(gobject_class,
+                           ARG_ORIENTATION,
+  g_param_spec_enum ("orientation",
+                     P_("Orientation"),
+                     P_("Orientation"),
+                     GTK_TYPE_ORIENTATION, 0,
+                     G_PARAM_READABLE|G_PARAM_WRITABLE));
 
   data_class->clone = gtk_plot_box_clone;
   data_class->draw_legend = gtk_plot_box_draw_legend;
@@ -113,38 +120,40 @@ gtk_plot_box_class_init (GtkPlotBoxClass *klass)
 }
 
 static void
-gtk_plot_box_set_arg (GtkObject      *object,
-                      GtkArg         *arg,
-                      guint           arg_id)
+gtk_plot_box_set_property (GObject      *object,
+                         guint prop_id,
+                         const GValue *value,
+                         GParamSpec *pspec)
 {
   GtkPlotBox *data;
 
   data = GTK_PLOT_BOX (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
       case ARG_ORIENTATION:
-        data->orientation  = GTK_VALUE_UINT(*arg);
+        data->orientation  = g_value_get_enum(value);
         break;
     }
 }
 
 static void
-gtk_plot_box_get_arg (GtkObject      *object,
-                      GtkArg         *arg,
-                      guint           arg_id)
+gtk_plot_box_get_property (GObject      *object,
+                         guint prop_id,
+                         GValue *value,
+                         GParamSpec *pspec)
 {
   GtkPlotBox *data;
 
   data = GTK_PLOT_BOX (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
       case ARG_ORIENTATION:
-        GTK_VALUE_UINT(*arg) = data->orientation;
+        g_value_set_enum(value, data->orientation);
         break;
       default:
-        arg->type = GTK_TYPE_INVALID;
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
     }
 }
@@ -376,7 +385,7 @@ gtk_plot_box_draw_legend(GtkPlotData *data, gint x, gint y)
 
   g_return_if_fail(data->plot != NULL);
   g_return_if_fail(GTK_IS_PLOT(data->plot));
-  g_return_if_fail(GTK_WIDGET_REALIZED(data->plot));
+  //g_return_if_fail(GTK_WIDGET_REALIZED(data->plot)); // is this required?? RRR
 
   plot = data->plot;
   area.x = GTK_WIDGET(plot)->allocation.x;
