@@ -41,9 +41,10 @@
 #include "gtkplotpc.h"
 #include "gtkplotps.h"
 
+
 static void gtk_plot_ps_class_init 		(GtkPlotPSClass *klass);
 static void gtk_plot_ps_init 			(GtkPlotPS *ps);
-static void gtk_plot_ps_finalize 		(GtkObject *object);
+static void gtk_plot_ps_destroy 		(GtkObject *object);
 /*********************************************************************/
 /* Postscript specific functions */
 static gboolean psinit				(GtkPlotPC *pc); 
@@ -162,11 +163,13 @@ static void
 gtk_plot_ps_class_init (GtkPlotPSClass *klass)
 {
   GtkObjectClass *object_class;
+  GObjectClass *gobject_class;
   GtkPlotPCClass *pc_class;
 
   parent_class = gtk_type_class (gtk_plot_pc_get_type ());
 
   object_class = (GtkObjectClass *) klass;
+  gobject_class = (GObjectClass *) klass;
   pc_class = (GtkPlotPCClass *) klass;
 
   pc_class->init = psinit;
@@ -190,17 +193,20 @@ gtk_plot_ps_class_init (GtkPlotPSClass *klass)
   pc_class->draw_string = psdrawstring;
   pc_class->draw_pixmap = psdrawpixmap;
 
-  object_class->finalize = gtk_plot_ps_finalize;
+  object_class->destroy = gtk_plot_ps_destroy;
 }
 
 static void
-gtk_plot_ps_finalize(GtkObject *object)
+gtk_plot_ps_destroy(GtkObject *object)
 {
   GtkPlotPS *ps;
 
   ps = GTK_PLOT_PS(object);
 
-  if(ps->psname) g_free(ps->psname);
+  if(ps->psname){
+    g_free(ps->psname);
+    ps->psname = NULL;
+  }
 }
 
 GtkObject *
@@ -356,8 +362,8 @@ static void pssetlineattr			(GtkPlotPC *pc,
     FILE *psout = GTK_PLOT_PS(pc)->psfile;
 
     fprintf(psout, "%g slw\n", line_width);
-    fprintf(psout, "%i slc\n", abs((int)cap_style - 1));
-    fprintf(psout, "%i slj\n", join_style);
+    fprintf(psout, "%d slc\n", abs(cap_style - 1));
+    fprintf(psout, "%d slj\n", join_style);
 
     if(line_style == 0)
             fprintf(psout,"[] 0 sd\n");  /* solid line */
@@ -446,7 +452,9 @@ psinit						(GtkPlotPC *pc)
     else
              fprintf(psout,"%%%%Orientation: Landscape\n");
 
+/*
     if(ps->epsflag)
+*/
           fprintf (psout,
                    "%%%%BoundingBox: 0 0 %d %d\n"
                    "%%%%Pages: 1\n"
@@ -575,19 +583,29 @@ psinit						(GtkPlotPC *pc)
     ps_reencode_font(psout, "Symbol");
     ps_reencode_font(psout, "ZapfChancery-MediumItalic");
     ps_reencode_font(psout, "ZapfDingbats");
-   
+    
+    fprintf(psout,"%%%%EndProlog\n%%%%BeginSetup\n"
+                "%%%%PageBoundingBox: 0 0 %d %d\n"
+                "%%%%PageOrientation: %s\n"
+                "%%%%PaperSize: %d %d\n",
+                 ps->page_width,
+                 ps->page_height, 
+                (ps->orientation == GTK_PLOT_PORTRAIT) ? "Portrait":"Landscape",
+                 ps->page_width,
+                 ps->page_height); 
+
     if(ps->orientation == GTK_PLOT_PORTRAIT)
-             fprintf(psout, "%d %d translate\n"
-                            "%g %g scale\n",
-                            0, ps->page_height,
-                            ps->scalex, -ps->scaley);
+             fprintf(psout, "%g %g scale\n",
+                            ps->scalex, ps->scaley);
 
     if(ps->orientation == GTK_PLOT_LANDSCAPE)
              fprintf(psout, "%g %g scale\n"
-                            "-90 rotate \n",
-                            ps->scalex, -ps->scaley);
+                            "90 rotate \n"
+                            "0 %d translate\n",
+                            ps->scalex, ps->scaley,
+                            -ps->page_height);
 
-    fprintf(psout,"%%%%EndProlog\n\n\n");
+    fprintf(psout,"%%%%EndSetup\n\n\n");
 
     return TRUE;
 }
@@ -627,6 +645,7 @@ psdrawpoint(GtkPlotPC *pc, gdouble x, gdouble y)
 {
   FILE *psout = GTK_PLOT_PS(pc)->psfile;
 
+  y = GTK_PLOT_PS(pc)->page_height - y;
   fprintf(psout, "n\n");
   fprintf(psout, "%g %g m\n", x, y);
   fprintf(psout, "%g %g l\n", x, y);
@@ -636,13 +655,13 @@ psdrawpoint(GtkPlotPC *pc, gdouble x, gdouble y)
 static void
 psdrawlines(GtkPlotPC *pc, GtkPlotPoint *points, gint numpoints)
 {
-  gint i;
+  gint i, page_height = GTK_PLOT_PS(pc)->page_height;
   FILE *psout = GTK_PLOT_PS(pc)->psfile;
  
   fprintf(psout,"n\n");
-  fprintf(psout,"%g %g m\n", points[0].x, points[0].y);
+  fprintf(psout,"%g %g m\n", points[0].x, page_height - points[0].y);
   for(i = 1; i < numpoints; i++)
-        fprintf(psout,"%g %g l\n", points[i].x, points[i].y);
+        fprintf(psout,"%g %g l\n", points[i].x, page_height - points[i].y);
 
   fprintf(psout,"s\n");
 }
@@ -650,13 +669,13 @@ psdrawlines(GtkPlotPC *pc, GtkPlotPoint *points, gint numpoints)
 static void
 psdrawpolygon(GtkPlotPC *pc, gboolean filled, GtkPlotPoint *points, gint numpoints)
 {
-  gint i;
+  gint i, page_height = GTK_PLOT_PS(pc)->page_height;
   FILE *psout = GTK_PLOT_PS(pc)->psfile;
 
   fprintf(psout,"n\n");
-  fprintf(psout,"%g %g m\n", points[0].x, points[0].y);
+  fprintf(psout,"%g %g m\n", points[0].x, page_height - points[0].y);
   for(i = 1; i < numpoints; i++)
-      fprintf(psout,"%g %g l\n", points[i].x, points[i].y);
+      fprintf(psout,"%g %g l\n", points[i].x, page_height - points[i].y);
 
   if(filled)
      fprintf(psout,"f\n");
@@ -670,8 +689,8 @@ static void psdrawline(GtkPlotPC *pc, gdouble x0, gdouble y0, gdouble xf, gdoubl
 {
   FILE *psout = GTK_PLOT_PS(pc)->psfile;
 
-  fprintf(psout, "%g %g m\n", x0, y0);
-  fprintf(psout, "%g %g l\n", xf, yf);
+  fprintf(psout, "%g %g m\n", x0, GTK_PLOT_PS(pc)->page_height - y0);
+  fprintf(psout, "%g %g l\n", xf, GTK_PLOT_PS(pc)->page_height - yf);
   fprintf(psout, "s\n");
 }
 
@@ -698,6 +717,7 @@ psdrawcircle(GtkPlotPC *pc, gboolean filled, gdouble x, gdouble y, gdouble size)
 {
   FILE *psout = GTK_PLOT_PS(pc)->psfile;
 
+  y = GTK_PLOT_PS(pc)->page_height - y;
   fprintf(psout,"n %g %g %g %g 0 360 ellipse\n", 
           x, y, size / 2., size / 2.);
 
@@ -715,8 +735,9 @@ psdrawellipse(GtkPlotPC *pc,
 {
   FILE *psout = GTK_PLOT_PS(pc)->psfile;
 
+  y = GTK_PLOT_PS(pc)->page_height - y;
   fprintf(psout,"n %g %g %g %g 0 360 ellipse\n", 
-          x+width/2., y+height/2., 
+          x+width/2., y-height/2., 
           width/2., height/2.);
 
   if(filled)
@@ -730,25 +751,26 @@ psoutputstring (GtkPlotPC *pc,
 		GtkPSFont *psfont,
 		GtkPSFont *latin_psfont,
 		gint height,
-		const GdkWChar *wstring,
+		const gchar *wstring,
 		const gchar *addstring)
 {
-  const GdkWChar *p;
-  GdkWChar wcs[2];
-  gint curcode = 0, code; /* 0..neither 1..latin 2..i18n */
+  const gchar *p;
+  gint curcode = 0;
   gchar begin[] = { 0, '(', '<' };
   gchar end[] = { 0, ')', '>' };
   GtkPSFont *fonts[3];
   FILE *out = GTK_PLOT_PS(pc)->psfile;
-  gchar *mbs, *c;
+  const gchar *c = NULL;
  
-  fonts[0] = NULL; 
-  fonts[1] = latin_psfont; 
-  fonts[2] = psfont; 
+  fonts[0] = NULL;
+  fonts[1] = latin_psfont;
+  fonts[2] = psfont;
 
   p = wstring;
   
   if (psfont->i18n_latinfamily) {
+    gint code;  /* 0..neither 1..latin 2..i18n */
+    gchar wcs[2];
     while (*p) {
       code = (*p >= 0 && *p <= 0x7f) ? 1 : 2;
       if (curcode && curcode != code)
@@ -762,7 +784,7 @@ psoutputstring (GtkPlotPC *pc,
 
       wcs[0] = *p++;
       wcs[1] = 0;
-      c = mbs = gdk_wcstombs(wcs);
+      c = wcs;
       
       if (code == 2) {
 	while (*c)
@@ -772,25 +794,27 @@ psoutputstring (GtkPlotPC *pc,
 	  fputc('\\', out);
 	fputc(*c, out);
       }
-      
-      g_free(mbs);
     }
   } else {
-    c = mbs = gdk_wcstombs(wstring);
+    c = wstring;
     
-    while (*c) {
-      if (curcode == 0) {
-	pssetfont(pc, psfont, height);
-	fputc(begin[1], out);
-	curcode = 1;
-      }
+    pssetfont(pc, psfont, height);
+    fputc(begin[1], out);
+    curcode = 1;
 
+    while (*c) {
+      const gchar *aux2 = c;
       if (*c == '(' || *c == ')')
 	fputc('\\', out);
-      fputc(*c++, out);
+     
+      if(++aux2 != g_utf8_next_char(c)){
+        fprintf(out, ") show <%02x> show (", (unsigned char)(*++c));
+        c++;
+      } else {
+        fputc(*c, out);
+        c = g_utf8_next_char(c);
+      } 
     }
-
-    g_free(mbs);
   }
 
   if (curcode)
@@ -827,7 +851,8 @@ psdrawstring	(GtkPlotPC *pc,
   gint twidth, theight, tdescent, tascent;
   gint tx, ty, width, height; 
   gint i;
-  GdkWChar *curstr, *wtext, *lastchar = NULL, bkspchar[2], *aux, *xaux;
+  const gchar *aux, *xaux, *wtext, *lastchar = NULL;
+  gchar *curstr, bkspchar[3];
   gchar num[4];
 
   if (text == NULL || strlen(text) == 0) return;
@@ -843,7 +868,7 @@ psdrawstring	(GtkPlotPC *pc,
 
   if (psfont->i18n_latinfamily) {
     latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily, italic,
-					     bold);
+					    bold);
   }
 
   gtk_plot_text_get_area(text, angle, justification, font, font_height,
@@ -851,6 +876,7 @@ psdrawstring	(GtkPlotPC *pc,
 
   tx += x;
   ty += y;
+
   if(!transparent){
     pssetcolor(pc, bg);
     gtk_plot_pc_draw_rectangle(pc,
@@ -890,14 +916,11 @@ psdrawstring	(GtkPlotPC *pc,
   gtk_plot_text_get_size(text, angle, psfont->psname, font_height, 
                          &twidth, &theight, &tascent, &tdescent);
 
-  if(angle == 90 || angle == 270) angle = 360 - angle;
   psgsave(pc);
-  fprintf(psout, "%d %d translate\n", x, y);
+  fprintf(psout, "%d %d translate\n", x, GTK_PLOT_PS(pc)->page_height - y);
   fprintf(psout, "%d rotate\n", angle);
 
   fprintf(psout, "0 0 m\n");
-  fprintf(psout, "1 -1 sc\n");
-
   
   if (psfont->i18n_latinfamily)
     special = TRUE;
@@ -905,7 +928,7 @@ psdrawstring	(GtkPlotPC *pc,
   c = text;
   while(c && *c != '\0' && *c != '\n') {
      if(*c == '\\'){
-         c++;
+         c = g_utf8_next_char(c);
          switch(*c){
            case '0': case '1': case '2': case '3':
            case '4': case '5': case '6': case '7': case '9':
@@ -917,7 +940,7 @@ psdrawstring	(GtkPlotPC *pc,
              break;
          }
      } else {
-         c++;
+         c = g_utf8_next_char(c);
      }
   }
 
@@ -960,17 +983,16 @@ psdrawstring	(GtkPlotPC *pc,
     return;
   }
 
-  i = strlen(text) + 2;
-  curstr = g_malloc0(sizeof(GdkWChar) * i);
-  aux = wtext = g_malloc0(sizeof(GdkWChar) * i);
-  gdk_mbstowcs(wtext, text, i - 1);
+  i = g_utf8_strlen(text, -1) + 2;
+  curstr = g_malloc0(sizeof(gchar) * i);
+  aux = wtext = text; 
 
   scale = font_height;
   curcnt = 0;
 
   while(aux && *aux != '\0' && *aux != '\n') {
      if(*aux == '\\'){
-         aux++;
+         aux = g_utf8_next_char(aux);
          switch(*aux){
            case '0': case '1': case '2': case '3':
            case '4': case '5': case '6': case '7': case '9':
@@ -979,7 +1001,7 @@ psdrawstring	(GtkPlotPC *pc,
 				 curstr, "show");
                   curcnt = 0;
                   psfont = gtk_psfont_get_by_family((gchar *)g_list_nth_data(family, *aux-'0'), italic, bold);
-		  aux++;
+		  aux = g_utf8_next_char(aux);
                   break;
            case '8':case 'g':
                   curstr[curcnt] = 0;
@@ -987,7 +1009,7 @@ psdrawstring	(GtkPlotPC *pc,
 				 curstr, "show");
                   curcnt = 0;
                   psfont = gtk_psfont_get_by_family("Symbol", italic, bold);
-                  aux++;
+                  aux = g_utf8_next_char(aux);
                   break;
            case 'B':
                   curstr[curcnt] = 0;
@@ -998,7 +1020,7 @@ psdrawstring	(GtkPlotPC *pc,
                   psfont = gtk_psfont_get_by_family(psfont->family, italic, bold);
 		  if (psfont->i18n_latinfamily)
 		    latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily, italic, bold);
-		  aux++;
+		  aux = g_utf8_next_char(aux);
                   break;
            case 'x':
                   xaux = aux + 1;
@@ -1009,7 +1031,7 @@ psdrawstring	(GtkPlotPC *pc,
 		      break;
                   }
                   if (i < 3){
-                     aux++;
+                     aux = g_utf8_next_char(aux);
                      break;
                   }
                   num[3] = '\0';
@@ -1034,7 +1056,7 @@ psdrawstring	(GtkPlotPC *pc,
                   psfont = gtk_psfont_get_by_family(psfont->family, italic, bold);
 		  if (psfont->i18n_latinfamily)
 		    latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily, italic, bold);
-		  aux++;
+		  aux = g_utf8_next_char(aux);
                   break;
            case 's':case '_':
                   curstr[curcnt] = 0;
@@ -1044,7 +1066,7 @@ psdrawstring	(GtkPlotPC *pc,
                   scale = 0.6 * font_height;
 		  offset -= (gint)scale / 2;
                   fprintf(psout, "0 %d rmoveto\n", -((gint)scale / 2));
-                  aux++;
+                  aux = g_utf8_next_char(aux);
                   break;
            case 'S':case '^':
                   curstr[curcnt] = 0;
@@ -1054,7 +1076,7 @@ psdrawstring	(GtkPlotPC *pc,
                   scale = 0.6 * font_height;
 		  offset += 0.5*font_height;
                   fprintf(psout, "0 %d rmoveto\n", (gint)(0.5*font_height));
-                  aux++;
+                  aux = g_utf8_next_char(aux);
                   break;
            case 'N':
                   curstr[curcnt] = 0;
@@ -1071,17 +1093,23 @@ psdrawstring	(GtkPlotPC *pc,
                   scale = font_height;
                   fprintf(psout, "0 %d rmoveto\n", -offset);
                   offset = 0;
-                  aux++;
+                  aux = g_utf8_next_char(aux);
                   break;
            case 'b':
                   curstr[curcnt] = '\0';
                   psoutputstring(pc, psfont, latin_psfont, (gint)scale,
 				 curstr, "show");
                   curcnt = 0;
-		  bkspchar[1] = 0;
                   if (lastchar) {
+                      const gchar *aux2 = lastchar;
                       bkspchar[0] = *lastchar;
-                      lastchar--;
+                      lastchar = g_utf8_prev_char(lastchar);
+		      bkspchar[1] = 0;
+                      if(--aux2 != lastchar){
+                        bkspchar[1] = *lastchar;
+                        lastchar = g_utf8_prev_char(lastchar);
+		        bkspchar[2] = 0;
+                      }
                   } else {
                       bkspchar[0] = 'X';
                       lastchar = NULL;
@@ -1089,7 +1117,7 @@ psdrawstring	(GtkPlotPC *pc,
 		  psoutputstring(pc, psfont, latin_psfont, (gint)scale,
 				 bkspchar,
 				 "stringwidth pop 0 exch neg exch rmoveto");
-                  aux++;
+                  aux = g_utf8_next_char(aux);
                   break;
            case '-':
                   curstr[curcnt] = 0;
@@ -1100,7 +1128,7 @@ psdrawstring	(GtkPlotPC *pc,
                   if (scale < 6) {
                       scale = 6;
                   }
-                  aux++;
+                  aux = g_utf8_next_char(aux);
                   break;
            case '+':
                   curstr[curcnt] = 0;
@@ -1108,21 +1136,27 @@ psdrawstring	(GtkPlotPC *pc,
 				 curstr, "show");
                   curcnt = 0;
                   scale += 3;
-                  aux++;
+                  aux = g_utf8_next_char(aux);
                   break;
            default:
                   if(aux && *aux != '\0' && *aux != '\n'){
                     curstr[curcnt++] = *aux;
-                    aux++;
+                    aux = g_utf8_next_char(aux);
                   }
                   break;
          }
      } else {
        if(aux && *aux != '\0' && *aux != '\n'){
-                curstr[curcnt++] = *aux;
-		lastchar = aux;
-                aux++;
-
+                const gchar *aux2 = aux;
+                if(g_utf8_next_char(aux) != ++aux2){
+                  curstr[curcnt++] = *aux++;
+//                  aux = g_utf8_next_char(aux);
+                  curstr[curcnt++] = *aux++;
+                } else {
+                  curstr[curcnt++] = *aux;
+		  lastchar = aux;
+                  aux = g_utf8_next_char(aux);
+                }
        }
      }
   }
@@ -1132,8 +1166,6 @@ psdrawstring	(GtkPlotPC *pc,
   psgrestore(pc);  
   fprintf(psout, "n\n");  
 
-
-  g_free(wtext);
   g_free(curstr);
 }
 
@@ -1191,7 +1223,7 @@ psclipmask(GtkPlotPC *pc, gdouble x, gdouble y, const GdkBitmap *mask)
   gint width, height;
   gint px, py;
   gint npoints = 0;
-  gint i;
+  gint i, page_height = GTK_PLOT_PS(pc)->page_height;
   GtkPlotVector *points;
   GdkImage *image;
 
@@ -1249,9 +1281,9 @@ psclipmask(GtkPlotPC *pc, gdouble x, gdouble y, const GdkBitmap *mask)
   fprintf(psout,"gsave\n");
 
   fprintf(psout,"n\n");
-  fprintf(psout,"%g %g m\n", x + points[0].x, y + points[0].y);
+  fprintf(psout,"%g %g m\n", x + points[0].x, page_height - y - points[0].y);
   for(i = 1; i < npoints; i++)
-      fprintf(psout,"%g %g l\n", x + points[i].x, y + points[i].y);
+      fprintf(psout,"%g %g l\n", x + points[i].x, page_height - y - points[i].y);
 
   fprintf(psout,"cp\n");
 
@@ -1272,9 +1304,14 @@ psclip(GtkPlotPC *pc, const GdkRectangle *clip)
   }
 
   fprintf(psout,"gsave\n");
-  fprintf(psout,"%d %d %d %d rectclip\n", clip->x, clip->y, clip->width, clip->height);
+  fprintf(psout,"%d %d %d %d rectclip\n", 
+                  clip->x,
+                  GTK_PLOT_PS(pc)->page_height - clip->y - clip->height,
+                  clip->width,  
+                  clip->height);
 }
 
+/* TODO: FIXME */
 
 static void 
 psdrawpixmap  (GtkPlotPC *pc,
@@ -1287,12 +1324,8 @@ psdrawpixmap  (GtkPlotPC *pc,
 {
   FILE *psout = GTK_PLOT_PS(pc)->psfile;
   GdkColormap *colormap;
-  GdkColorContext *cc;
-  GdkVisual *visual;
 
   colormap = gdk_colormap_get_system ();
-  visual = gdk_visual_get_system ();
-  cc = gdk_color_context_new(visual, colormap);
 
   fprintf(psout, "gsave\n");
   if(pixmap){
@@ -1304,6 +1337,7 @@ psdrawpixmap  (GtkPlotPC *pc,
                           width, height);
 
     if(mask) gtk_plot_pc_clip_mask(pc, xdest, ydest, mask);
+    ydest = GTK_PLOT_PS(pc)->page_height - ydest - height;
 
     fprintf(psout, "%d %g translate\n", xdest, ydest + height * scale_y);
     fprintf(psout, "%g %g scale\n",width * scale_x, height * scale_y);
@@ -1312,15 +1346,13 @@ psdrawpixmap  (GtkPlotPC *pc,
     fprintf(psout, "{ currentfile scanline readhexstring pop } false 3\n");
     fprintf(psout, "colorimage\n");
 
-
-    for(y = 0; y < height; y++){
+    for(y = height - 1; y >= 0; y--){
       for(x = 0; x < width; x++){
         GdkColor color;
         gchar string[7];
 
         color.pixel = gdk_image_get_pixel(image, x, y);
-        gdk_color_context_query_color(cc, &color);
-
+	gdk_colormap_query_color(colormap, color.pixel, &color);
         color_to_hex(color, string);
         fprintf(psout,"%s",string);
         if(fmod(x + 1, 13) == 0) fprintf(psout, "\n");
@@ -1332,7 +1364,6 @@ psdrawpixmap  (GtkPlotPC *pc,
     if(mask) gtk_plot_pc_clip_mask(pc, xdest, ydest, NULL);
   }
 
-  gdk_color_context_free(cc);
   fprintf(psout, "grestore\n");
 }
 
