@@ -1,5 +1,5 @@
 /* gtkpsfont - PostScript Fonts handling
- * Copyright 1999-2001  Adrian E. Feiguin <feiguin@ifir.edu.ar>
+ * Copyright 1999-2006  Adrian E. Feiguin <feiguin@ifir.edu.ar>
  *
  * Some code borrowed from
  * DiaCanvas -- a technical canvas widget
@@ -28,350 +28,282 @@
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <string.h>
+#ifndef G_PLATFORM_WIN32
+#include <fontconfig/fontconfig.h>
+#endif //G_PLATFORM_WIN32
+#include <pango/pango.h>
 #include "gtkpsfont.h"
 
 #define FONTCACHE_SIZE 17
 #define NUM_X11_FONTS 2
 
-
-static GtkPSFont font_data[] = 
+static GtkPSFont font_data[] =
 {
   { "Times-Roman",
     "Times-Roman",
     "Times-Roman",
-    { "-adobe-times-medium-r-normal",
-      NULL
-    },
+    "Nimbus Roman No9 L, Regular",
     NULL,
     FALSE, FALSE
-  }, 
+  },
   { "Times-Italic",
     "Times-Italic",
     "Times-Roman",
-    { "-adobe-times-medium-i-normal",
-      NULL
-    },
+    "Nimbus Roman No9 L, Italic",
     NULL,
     TRUE, FALSE
-  }, 
+  },
   { "Times-Bold",
     "Times-Bold",
     "Times-Roman",
-    { "-adobe-times-bold-r-normal",
-      NULL
-    },
+    "Nimbus Roman No9 L, Bold",
     NULL,
     FALSE, TRUE
-  }, 
+  },
   { "Times-BoldItalic",
     "Times-BoldItalic",
     "Times-Roman",
-    { "-adobe-times-bold-i-normal",
-      NULL
-    },
+    "Nimbus Roman No9 L, Bold Italic",
     NULL,
     TRUE, TRUE
-  }, 
+  },
   { "AvantGarde-Book",
     "AvantGarde-Book",
     "AvantGarde",
-    { "-adobe-avantgarde-book-r-normal",
-      "-schumacher-clean-medium-r-normal"
-    },
+    "URW Gothic L, Book",
     NULL,
     FALSE, FALSE
   },
   { "AvantGarde-BookOblique",
     "AvantGarde-BookOblique",
     "AvantGarde",
-    { "-adobe-avantgarde-book-o-normal",
-      "-schumacher-clean-medium-i-normal"
-    },
+    "URW Gothic L, Book Oblique",
     NULL,
     TRUE, FALSE
   },
   { "AvantGarde-Demi",
     "AvantGarde-Demi",
     "AvantGarde",
-    { "-adobe-avantgarde-demibold-r-normal",
-      "-schumacher-clean-bold-r-normal"
-    },
+    "URW Gothic L, Demi",
     NULL,
     FALSE, TRUE
   },
   { "AvantGarde-DemiOblique",
     "AvantGarde-DemiOblique",
     "AvantGarde",
-    { "-adobe-avantgarde-demibold-o-normal",
-      "-schumacher-clean-bold-i-normal"
-    },
+    "URW Gothic L, Demi Oblique",
     NULL,
     TRUE, TRUE
   },
   { "Bookman-Light",
     "Bookman-Light",
     "Bookman",
-    { "-adobe-bookman-light-r-normal",
-      "-adobe-times-medium-r-normal"
-    },
+    "URW Bookman L, Light",
     NULL,
     FALSE, FALSE
   },
   { "Bookman-LightItalic",
     "Bookman-LightItalic",
     "Bookman",
-    { "-adobe-bookman-light-i-normal",
-      "-adobe-times-medium-i-normal"
-    },
+    "URW Bookman L, Light Italic",
     NULL,
     TRUE, FALSE
   },
   { "Bookman-Demi",
     "Bookman-Demi",
     "Bookman",
-    { "-adobe-bookman-demibold-r-normal",
-      "-adobe-times-bold-r-normal"
-    },
+    "URW Bookman L, Demi Bold",
     NULL,
     FALSE, TRUE
   },
   { "Bookman-DemiItalic",
     "Bookman-DemiItalic",
     "Bookman",
-    { "-adobe-bookman-demibold-i-normal",
-      "-adobe-times-bold-i-normal"
-    },
+    "URW Bookman L, Demi Bold Italic",
     NULL,
     TRUE, TRUE
   },
   { "Courier",
     "Courier",
     "Courier",
-    { "-adobe-courier-medium-r-normal",
-      NULL
-    },
+    "Nimbus Mono L, Regular",
     NULL,
     FALSE, FALSE
   },
   { "Courier-Oblique",
     "Courier-Oblique",
     "Courier",
-    { "-adobe-courier-medium-o-normal",
-      NULL
-    },
+    "Nimbus Mono L, Regular Oblique",
     NULL,
     TRUE, FALSE
   },
   { "Courier-Bold",
     "Courier-Bold",
     "Courier",
-    { "-adobe-courier-bold-r-normal",
-      NULL
-    },
+    "Nimbus Mono L, Bold",
     NULL,
     FALSE, TRUE
   },
   { "Courier-BoldOblique",
     "Courier-BoldOblique",
     "Courier",
-    { "-adobe-courier-bold-o-normal",
-      NULL
-    },
+    "Nimbus Mono L, Bold Oblique",
     NULL,
     TRUE, TRUE
   },
   { "Helvetica",
     "Helvetica",
     "Helvetica",
-    { "-adobe-helvetica-medium-r-normal",
-      NULL
-    },
+    "Sans, Regular",
     NULL,
     FALSE, FALSE
   },
   { "Helvetica-Oblique",
     "Helvetica-Oblique",
     "Helvetica",
-    { "-adobe-helvetica-medium-o-normal",
-      NULL
-    },
+    "Sans, Regular Italic",
     NULL,
     TRUE, FALSE
   },
   { "Helvetica-Bold",
     "Helvetica-Bold",
     "Helvetica",
-    { "-adobe-helvetica-bold-r-normal",
-      NULL
-    },
+    "Sans, Bold",
     NULL,
     FALSE, TRUE
   },
   { "Helvetica-BoldOblique",
     "Helvetica-BoldOblique",
     "Helvetica",
-    { "-adobe-helvetica-bold-o-normal",
-      NULL
-    },
+    "Sans, Bold Italic",
     NULL,
     TRUE, TRUE
   },
   { "Helvetica-Narrow",
     "Helvetica-Narrow",
     "Helvetica-Narrow",
-    { "-adobe-helvetica-medium-r-normal",
-      NULL
-    },
+    "Sans, Regular Condensed",
     NULL,
     FALSE, FALSE
   },
   { "Helvetica-Narrow-Oblique",
     "Helvetica-Narrow-Oblique",
     "Helvetica-Narrow",
-    { "-adobe-helvetica-medium-o-normal",
-      NULL
-    },
+    "Sans, Regular Condensed Italic",
     NULL,
     TRUE, FALSE
   },
   { "Helvetica-Narrow-Bold",
     "Helvetica-Narrow-Bold",
     "Helvetica-Narrow",
-    { "-adobe-helvetica-bold-r-normal",
-      NULL
-    },
+    "Sans, Bold Condensed",
     NULL,
     FALSE, TRUE
   },
   { "Helvetica-Narrow-BoldOblique",
     "Helvetica-Narrow-BoldOblique",
     "Helvetica-Narrow",
-    { "-adobe-helvetica-bold-o-normal",
-      NULL
-    },
+    "Sans, Bold Condensed Italic",
     NULL,
     TRUE, TRUE
   },
   { "NewCenturySchoolbook-Roman",
     "NewCenturySchlbk-Roman",
     "NewCenturySchlbk",
-    { "-adobe-new century schoolbook-medium-r-normal",
-      NULL
-    },
+    "Century Schoolbook L, Roman",
     NULL,
     FALSE, FALSE
   },
   { "NewCenturySchoolbook-Italic",
     "NewCenturySchlbk-Italic",
     "NewCenturySchlbk",
-    { "-adobe-new century schoolbook-medium-i-normal",
-      NULL
-    },
+    "Century Schoolbook L, Italic",
     NULL,
     TRUE, FALSE
   },
   { "NewCenturySchoolbook-Bold",
     "NewCenturySchlbk-Bold",
     "NewCenturySchlbk",
-    { "-adobe-new century schoolbook-bold-r-normal",
-      NULL
-    },
+    "Century Schoolbook L, Bold",
     NULL,
     FALSE, TRUE
   },
   { "NewCenturySchoolbook-BoldItalic",
     "NewCenturySchlbk-BoldItalic",
     "NewCenturySchlbk",
-    { "-adobe-new century schoolbook-bold-i-normal",
-      NULL
-    },
+    "Century Schoolbook L, Bold Italic",
     NULL,
     TRUE, TRUE
   },
   { "Palatino-Roman",
     "Palatino-Roman",
-    "Lucida",
-    { "-adobe-palatino-medium-r-normal",
-      "-*-lucidabright-medium-r-normal"
-    },
+    "Palatino",
+    "URW Palladio L, Roman",
     NULL,
     FALSE, FALSE
   },
   { "Palatino-Italic",
     "Palatino-Italic",
-    "Lucida",
-    { "-adobe-palatino-medium-i-normal",
-      "-*-lucidabright-medium-i-normal"
-    },
+    "Palatino",
+    "URW Palladio L, Italic",
     NULL,
     TRUE, FALSE
   },
   { "Palatino-Bold",
     "Palatino-Bold",
-    "Lucida",
-    { "-adobe-palatino-bold-r-normal",
-      "-*-lucidabright-demibold-r-normal"
-    },
+    "Palatino",
+    "URW Palladio L, Bold",
     NULL,
     FALSE, TRUE
   },
   { "Palatino-BoldItalic",
     "Palatino-BoldItalic",
-    "Lucida",
-    { "-adobe-palatino-bold-i-normal",
-      "-*-lucidabright-demibold-i-normal"
-    },
+    "Palatino",
+    "URW Palladio L, Bold Italic",
     NULL,
     TRUE, TRUE
   },
   { "Symbol",
     "Symbol",
     "Symbol",
-    {
-      "-adobe-symbol-medium-r-normal",
-      "-*-symbol-medium-r-normal"
-    },
+    "Standard Symbols L, Regular",
     NULL,
     FALSE, FALSE
   },
   { "ZapfChancery-MediumItalic",
     "ZapfChancery-MediumItalic",
     "ZapfChancery",
-    { "-adobe-zapf chancery-medium-i-normal",
-      "-*-itc zapf chancery-medium-i-normal"
-    },
+    "URW Chancery L, Medium Italic",
     NULL,
     FALSE, FALSE
   },
   { "ZapfDingbats",
     "ZapfDingbats",
     "ZapfDingbats",
-    { "-adobe-zapf dingbats-medium-r-normal",
-      "-*-itc zapf dingbats-*-*-*"
-    },
+    "Dingbats, Regular",
     NULL,
     FALSE, FALSE
   },
 };
 
+
 #define NUM_FONTS (sizeof(font_data)/sizeof(GtkPSFont))
 
-gchar *last_resort_fonts[] = {
-  "-adobe-courier-medium-r-normal",
-  "fixed" /* Must be last. This is guaranteed to exist on an X11 system. */
-};
-
-#define NUM_LAST_RESORT_FONTS 2
-
 static GList *user_fonts;
 static gboolean psfont_init = FALSE;
 static GList *psfont_families;
 static gint numf;
 static gint psfont_refcount = 0;
 
+#ifdef G_PLATFORM_WIN32
+static const char *default_font = "sans";
+#else
+static const char *default_font = "fixed";
+#endif
+
 static GtkPSFont *find_psfont		(const gchar *name);
 
-gint 
+gint
 gtk_psfont_init()
 {
   GtkPSFont *data = NULL;
@@ -389,16 +321,6 @@ gtk_psfont_init()
 */
 
   psfont_init = TRUE;
-/*
-  for(i = 0; i < NUM_FONTS; i++){
-    gtk_psfont_add_font(font_data[i].fontname,
-			font_data[i].psname,
-			font_data[i].family,
-			font_data[i].xfont,
-			font_data[i].italic,
-			font_data[i].bold);
-  }
-*/  
   psfont_families = NULL;
   numf = 0;
 
@@ -411,7 +333,7 @@ gtk_psfont_init()
     if(new_family){
          numf = numf + 1;
          psfont_families = g_list_append(psfont_families, font_data[i].family);
-    }     
+    }
   }
 
   fonts = user_fonts;
@@ -419,13 +341,13 @@ gtk_psfont_init()
     data = (GtkPSFont *) fonts->data;
     new_family = TRUE;
     for(j = 0; j < numf; j++){
-       if(strcmp(data->family, (gchar *)g_list_nth_data(psfont_families, j)) == 0) 
+       if(strcmp(data->family, (gchar *)g_list_nth_data(psfont_families, j)) == 0)
          new_family = FALSE;
     }
     if(new_family){
          numf = numf + 1;
          psfont_families = g_list_append(psfont_families, data->family);
-    }     
+    }
     fonts = fonts->next;
   }
 
@@ -433,7 +355,7 @@ gtk_psfont_init()
 }
 
 
-void 
+void
 gtk_psfont_unref()
 {
   GList *list;
@@ -469,11 +391,12 @@ gtk_psfont_get_by_name(const gchar *name)
   font = find_psfont(name);
 
   if (font == NULL) {
-    font = find_psfont("Courier");
+    font = find_psfont(default_font);
     if (font == NULL) {
-      g_warning ("Error, couldn't locate font. Shouldn't happend.");
+      g_warning ("Error, couldn't locate default font. Shouldn't happen.");
     } else {
-      g_message ("Font %s not found, using Courier instead.", name);
+      g_message ("Postscript font %s not found, using %s instead.",
+		 name, default_font);
     }
   }
 
@@ -483,105 +406,154 @@ gtk_psfont_get_by_name(const gchar *name)
 GdkFont *
 gtk_psfont_get_gdkfont(GtkPSFont *font, gint height)
 {
-  GdkFont *gdk_font = NULL;
-  gchar *x11_font;
-  gint bufsize;
-  gchar *buffer = NULL;
-  gint i;
-  gint auxheight;
-  gint min_height = 8;
+  PangoFontDescription *font_desc;
+  GdkFont *gdkfont;
+
+  g_return_val_if_fail (font != NULL, NULL);
 
   if (height <= 0) height = 1;
- 
-  for (i = 0; i < NUM_X11_FONTS; i++) {
-    x11_font = font->xfont[i];
 
-    if (x11_font != NULL) {
-     bufsize = strlen(x11_font)+25;  /* Should be enough */
-     buffer = (gchar *)g_malloc(bufsize);
+  font_desc = gtk_psfont_get_font_description(font, height);
+  gdkfont = font_desc ? gdk_font_from_description(font_desc) : NULL;
+  if (font_desc)
+    pango_font_description_free(font_desc);
 
-     for(auxheight = MAX(height, min_height); auxheight >= min_height; auxheight--){
-       if (font->i18n_latinfamily) {
-	 g_snprintf(buffer, bufsize, "%s-*-%d-*-*-*-*-*-*-*,*", x11_font, auxheight);
-	 gdk_font = gdk_fontset_load(buffer);
-       } else {
-	 g_snprintf(buffer, bufsize, "%s-*-%d-*-*-*-*-*-*-*", x11_font, auxheight);
-	 gdk_font = gdk_font_load(buffer);
-       }
-       
-      if (gdk_font != NULL) {
-         g_free(buffer);
-         break;
-      }
-     }
+  if (!gdkfont) {
+    font_desc = gtk_psfont_get_font_description(find_psfont(default_font), height);
+    gdkfont = font_desc ? gdk_font_from_description(font_desc) : NULL;
+    if (font_desc)
+      pango_font_description_free(font_desc);
 
-     if(gdk_font != NULL) break;
-    }
-
-    g_free(buffer);
+    if (gdkfont)
+      g_message ("Pango font %s %i (PS font %s) not found, using %s instead.",
+		 font->pango_description, height, font->fontname, default_font);
+    else
+      g_warning ("Error, couldn't locate default font. Shouldn't happen.");
   }
 
-  if (gdk_font == NULL) {
-    for (i=0; i < NUM_LAST_RESORT_FONTS; i++) {
-      x11_font = last_resort_fonts[i];
-      bufsize = strlen(x11_font)+25;  /* Should be enought*/
-      buffer = (char *)g_malloc(bufsize);
-      
-      for(auxheight = MAX(height, min_height); auxheight >= min_height; auxheight--){
-       g_snprintf(buffer, bufsize, "%s-*-%d-*-*-*-*-*-*-*", x11_font, auxheight);
-    
-       gdk_font = gdk_font_load(buffer);
-        if (gdk_font != NULL) {
-          g_free(buffer);
-          break;
-       }
-      }
+  return gdkfont;
+}
 
-      if (gdk_font != NULL) {
-	g_warning("Could not find X Font for %s, using %s instead.",
-		  font->fontname, x11_font);
-	break;
-      }
+PangoFontDescription *
+gtk_psfont_get_font_description(GtkPSFont *font, gint height)
+{
+  PangoFontDescription *font_desc;
+  gchar *font_string;
+  GtkSettings *settings = gtk_settings_get_for_screen(gdk_screen_get_default());
+  GObjectClass *klass;
+  gdouble dpi;
 
-      g_free(buffer);
-    }
+  g_return_val_if_fail (font != NULL, NULL);
+
+  if (height <= 0) height = 1;
+
+/* Dirty hack to get the correct font size for this device
+http://mail.gnome.org/archives/gtk-i18n-list/2003-August/msg00001.html
+*/
+
+  klass = G_OBJECT_CLASS(GTK_SETTINGS_GET_CLASS(settings));
+/* Check that the properties we're looking at are defined. */
+  if (!g_object_class_find_property(klass, "gtk-xft-dpi")) {
+    dpi = 96.;
+  } else {
+    /* Read the settings. */
+    gint int_dpi;
+    g_object_get(G_OBJECT(settings),
+    	         "gtk-xft-dpi", &int_dpi,
+	         NULL);
+    if(int_dpi <= 0)
+      dpi = 96;
+    else
+      dpi = int_dpi / PANGO_SCALE;
   }
 
-  if (gdk_font == NULL) 
-	g_warning("Could not find X Font for %s", font->fontname);
-	
-  return gdk_font;
+/*
+{
+    GdkScreen *screen = gdk_screen_get_default ();
+    FcPattern *pattern;
+
+    pattern = FcPatternCreate();
+    if (pattern)
+      {
+        XftDefaultSubstitute (GDK_SCREEN_XDISPLAY (screen),
+                              GDK_SCREEN_XNUMBER (screen),
+                              pattern);
+        FcPatternGetDouble (pattern, FC_DPI, 0, &dpi);
+        FcPatternDestroy (pattern);
+      }
+}
+*/
+  height *= 75./dpi;
+
+  font_string = g_strdup_printf("%s %i", font->pango_description, height);
+  font_desc = pango_font_description_from_string(font_string);
+  g_free(font_string);
+
+  if (!font_desc) {
+    font_string = g_strdup_printf("%s %i", default_font, height);
+    font_desc = pango_font_description_from_string(font_string);
+    g_free(font_string);
+    if (font_desc)
+      g_message ("Font %s not describable, using %s instead.",
+		 font->fontname, default_font);
+    else
+      g_warning ("Error, couldn't describe default font. Shouldn't happen.");
+  }
+
+  /* Loading via the pango fontset facility means that pango.aliases is used */
+/* This is not working */
+/* It is screwing up the whole thing */
+/*
+  if (font_desc) {
+    PangoContext *context = gdk_pango_context_get();
+    PangoFontset *pffontset;
+    PangoFont *pffont;
+
+    pffontset = pango_context_load_fontset(context, font_desc,
+    pango_context_get_language(context));
+    if (pffontset) {
+      pffont = pango_fontset_get_font(pffontset, g_utf8_get_char(" "));
+      if (pffont) {
+        PangoFontDescription *font_desc;
+	desc = pango_font_describe(pffont);
+	g_object_unref(G_OBJECT(pffont));
+	if (desc) {
+	  pango_font_description_free(font_desc);
+	  font_desc = desc;
+	}
+      }
+      g_object_unref(G_OBJECT(pffontset));
+    }
+  }
+*/
+  return font_desc;
 }
 
 
-gchar *
-gtk_psfont_get_psfontname(const gchar *fontname)
+const gchar *
+gtk_psfont_get_psfontname(GtkPSFont *font)
 {
-  GtkPSFont *font = NULL;
- 
-  font = find_psfont(fontname); 
-  if(!font) 
-     font = find_psfont("Courier");  
+
+  g_return_val_if_fail (font != NULL, NULL);
 
   return font->psname;
 }
 
 void
-gtk_psfont_add_font (const gchar *fontname, 
+gtk_psfont_add_font (const gchar *fontname,
                      const gchar *psname, const gchar *family,
-                     gchar *x_string[], 
+                     const gchar *pango_description,
                      gboolean italic, gboolean bold)
 {
   GtkPSFont *font;
 
-  font = g_new(GtkPSFont, 1);
+  font = g_new0(GtkPSFont, 1);
 
-  font->fontname = g_strdup(fontname); 
-  font->psname = g_strdup(psname); 
-  font->family = g_strdup(family); 
+  font->fontname = g_strdup(fontname);
+  font->psname = g_strdup(psname);
+  font->family = g_strdup(family);
+  font->pango_description = g_strdup(pango_description);
   font->i18n_latinfamily = NULL;
-  font->xfont[0] = g_strdup(x_string[0]);
-  font->xfont[1] = g_strdup(x_string[1]);
   font->italic = italic;
   font->bold = bold;
   font->vertical = FALSE;
@@ -590,21 +562,21 @@ gtk_psfont_add_font (const gchar *fontname,
 }
 
 void
-gtk_psfont_add_i18n_font (const gchar *fontname, 
+gtk_psfont_add_i18n_font (const gchar *fontname,
                          const gchar *psname, const gchar *family,
-                         const gchar *i18n_latinfamily, gchar *x_string[], 
+                         const gchar *i18n_latinfamily,
+                         const gchar *pango_description,
                          gboolean italic, gboolean bold, gboolean vertical)
 {
   GtkPSFont *font;
 
-  font = g_new(GtkPSFont, 1);
+  font = g_new0(GtkPSFont, 1);
 
-  font->fontname = g_strdup(fontname); 
-  font->psname = g_strdup(psname); 
-  font->family = g_strdup(family); 
+  font->fontname = g_strdup(fontname);
+  font->psname = g_strdup(psname);
+  font->family = g_strdup(family);
+  font->pango_description = g_strdup(pango_description);
   font->i18n_latinfamily = g_strdup(i18n_latinfamily);
-  font->xfont[0] = g_strdup(x_string[0]);
-  font->xfont[1] = g_strdup(x_string[1]);
   font->italic = italic;
   font->bold = bold;
   font->vertical = vertical;
@@ -642,17 +614,17 @@ find_psfont(const gchar *name)
 
   if(fontdata == NULL) {
     for(i = 0; i < NUM_FONTS; i++){
-      if(strcmp(name, font_data[i].fontname) == 0) { 
+      if(strcmp(name, font_data[i].fontname) == 0) {
 	fontdata = &font_data[i];
 	break;
       }
-      if(strcmp(name, font_data[i].psname) == 0) { 
+      if(strcmp(name, font_data[i].psname) == 0) {
 	fontdata = &font_data[i];
        break;
       }
     }
   }
-  
+
   return fontdata;
 }
 
@@ -682,10 +654,10 @@ gtk_psfont_get_by_family(const gchar *name, gboolean italic, gboolean bold)
     }
     fonts = fonts->next;
   }
-    
+
   if(fontdata == NULL) {
     for(i = 0; i < NUM_FONTS; i++){
-      if(strcmp(name, font_data[i].family) == 0) { 
+      if(strcmp(name, font_data[i].family) == 0) {
 	return_data = &font_data[i];
 	if(font_data[i].italic == italic && font_data[i].bold == bold){
 	  fontdata = &font_data[i];
@@ -724,7 +696,7 @@ gtk_psfont_get_char_size(GtkPSFont *psfont,
 {
   GdkFont *dfont;
   gint w, a, d, w0;
- 
+
   if (psfont->i18n_latinfamily && psfont->vertical && (0 > wc || wc > 0x7f)) {
     /* vertical-writing CJK postscript fonts. */
     w = (font->ascent + font->descent);
@@ -736,7 +708,6 @@ gtk_psfont_get_char_size(GtkPSFont *psfont,
       dfont = latin_font;
     else
       dfont = font;
-
     w = gdk_char_width_wc(dfont, wc);
     a = dfont->ascent;
     d = dfont->descent;
