@@ -25,6 +25,8 @@ GtkPlot3D *plot3d;
 gint nlayers = 0;
 GtkWidget *active_plot;
 
+static void activate_plot(GtkWidget *widget, gpointer data);
+
 void
 quit ()
 {
@@ -95,31 +97,12 @@ select_item(GtkWidget *widget, GdkEvent *event, GtkPlotCanvasChild *child,
 
     widget_list = plots;
     active_widget = GTK_WIDGET(GTK_PLOT_CANVAS_PLOT(child)->plot);
-    n = 0;
-                                                                                
-    while(n < nlayers)
-      {
-        if(widget_list[n] == active_widget){
-            active_plot = plots[n];
-            GTK_BUTTON(buttons[n])->button_down = TRUE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
-        }else{
-            GTK_BUTTON(buttons[n])->button_down = FALSE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
-        }
-        gtk_widget_queue_draw(buttons[n]);
-                                                                                
-        n++;
-      }
-
   }
 
   return TRUE;
 }
 
-gint
+static void
 activate_plot(GtkWidget *widget, gpointer data)
 {
   GtkWidget **widget_list = NULL;
@@ -133,28 +116,24 @@ activate_plot(GtkWidget *widget, gpointer data)
 
   while(n < nlayers)
     {
+      gtk_signal_handler_block_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
       if(widget_list[n] == active_widget){
-            active_plot = plots[n];  
-            GTK_BUTTON(buttons[n])->button_down = TRUE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
+            active_plot = plots[n];
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), TRUE);
       }else{
-            GTK_BUTTON(buttons[n])->button_down = FALSE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), FALSE);
       }
-      gtk_widget_queue_draw(buttons[n]);
-      
+      gtk_signal_handler_unblock_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
       n++;
     }
-
-  return TRUE;
 }
 
 GtkWidget *
 new_layer(GtkWidget *canvas, GtkType type)
 {
  gchar label[10];
+ GtkRequisition req;
+ gint size;
 
  nlayers++;
 
@@ -166,16 +145,20 @@ new_layer(GtkWidget *canvas, GtkType type)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
- gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
+ gtk_widget_size_request(buttons[nlayers-1], &req);
+ size = MAX(req.width,req.height);
+ gtk_widget_set_usize(buttons[nlayers-1], size, size);
+ gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*size, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
+
  if(type == GTK_TYPE_PLOT3D)
    plots[nlayers-1] = gtk_plot3d_new_with_size(NULL, .50, .50);
  else
    plots[nlayers-1] = gtk_plot_new_with_size(NULL, .40, .40);
 
  gtk_widget_show(plots[nlayers-1]);
+ gtk_signal_connect(GTK_OBJECT(buttons[nlayers-1]), "toggled", GTK_SIGNAL_FUNC(activate_plot), NULL);
 
  return plots[nlayers-1];
 }
@@ -323,6 +306,7 @@ int main(int argc, char *argv[]){
  gtk_widget_show(surface);
 
  gtk_plot_data_set_gradient(GTK_PLOT_DATA(surface), .2, .8, 6, 0);
+ gtk_plot_data_set_gradient(GTK_PLOT_DATA(surface), 0.00001, 0.99999, 10, 0);
 /* 
  gtk_plot_surface_set_transparent(GTK_PLOT_SURFACE(surface), TRUE);
 */
