@@ -29,6 +29,8 @@ int main(int argc, char *argv[]){
  GtkWidget *scrollw2;
  GtkWidget *iconlist1;
  GtkWidget *iconlist2;
+ GdkColor color;
+ GtkIconListItem *item;
  gchar label[10];
  gint i;
  
@@ -77,7 +79,7 @@ int main(int argc, char *argv[]){
 
 
  iconlist1 = gtk_icon_list_new(48, GTK_ICON_LIST_TEXT_BELOW);
- gtk_icon_list_set_selection_mode(GTK_ICON_LIST(iconlist1), 0);
+ gtk_icon_list_set_selection_mode(GTK_ICON_LIST(iconlist1), GTK_SELECTION_SINGLE);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw1),iconlist1);
  for(i=0; i<20; i++){
   sprintf(label,"Data %d",i); 
@@ -88,15 +90,23 @@ int main(int argc, char *argv[]){
  gtk_icon_list_set_selection_mode(GTK_ICON_LIST(iconlist2), GTK_SELECTION_MULTIPLE);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw2),iconlist2);
  for(i=0; i<20; i++){
+  GtkIconListItem *new_item;
   sprintf(label,"Plot %d",i); 
-  gtk_icon_list_add_from_data(GTK_ICON_LIST(iconlist2), plot_icon2, label, NULL);
+  new_item = gtk_icon_list_add_from_data(GTK_ICON_LIST(iconlist2), plot_icon2, label, NULL);
+  if(i==0) item = new_item;
  }
 
+ gdk_color_parse("light blue",&color);
+ gdk_color_alloc(gdk_colormap_get_system(), &color);
+ gtk_icon_list_set_background(GTK_ICON_LIST(iconlist1), &color);
+
  gtk_widget_show(iconlist1);
  gtk_widget_show(iconlist2);
 
+ gtk_icon_list_set_active_icon(GTK_ICON_LIST(iconlist2), item);
  gtk_widget_show(window1);
 
+
 /*
  gtk_signal_connect(GTK_OBJECT(iconlist),"button_press_event", clear, NULL);
 */
