@@ -128,25 +128,6 @@ select_item(GtkWidget *widget, GdkEvent *event, GtkPlotCanvasChild *child,
 
     widget_list = plots;
     active_widget = GTK_WIDGET(GTK_PLOT_CANVAS_PLOT(child)->plot);
-    n = 0;
-                                                                                
-    while(n < nlayers)
-      {
-        if(widget_list[n] == active_widget){
-            active_plot = plots[n];
-            GTK_BUTTON(buttons[n])->button_down = TRUE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
-        }else{
-            GTK_BUTTON(buttons[n])->button_down = FALSE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
-        }
-        gtk_widget_queue_draw(buttons[n]);
-                                                                                
-        n++;
-      }
-
   }
 
   return TRUE;
@@ -164,24 +145,22 @@ activate_plot(GtkWidget *widget, gpointer data)
   widget_list = buttons;
   active_widget = widget;
 
+
   while(n < nlayers)
     {
+      gtk_signal_handler_block_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
       if(widget_list[n] == active_widget){
             active_plot = plots[n];  
-            GTK_BUTTON(buttons[n])->button_down = TRUE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), TRUE);
       }else{
-            GTK_BUTTON(buttons[n])->button_down = FALSE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), FALSE);
       }
-      gtk_widget_queue_draw(buttons[n]);
+      gtk_signal_handler_unblock_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
       
       n++;
     }
 
-  return TRUE;
+  return FALSE;
 }
 
 
@@ -189,6 +168,8 @@ GtkWidget *
 new_layer(GtkWidget *canvas)
 {
  gchar label[10];
+ GtkRequisition req;
+ gint size;
 
  nlayers++;
 
@@ -200,8 +181,10 @@ new_layer(GtkWidget *canvas)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
- gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
+ gtk_widget_size_request(buttons[nlayers-1], &req);
+ size = MAX(req.width,req.height);
+ gtk_widget_set_usize(buttons[nlayers-1], size, size); 
+ gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*size, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
  gtk_signal_connect(GTK_OBJECT(buttons[nlayers-1]), "toggled",
@@ -479,7 +462,7 @@ int main(int argc, char *argv[]){
                           "You can use \\ssubscripts\\b\\b\\b\\b\\b\\b\\b\\b\\b\\b\\N\\Ssuperscripts");
  gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .40, .720, .0, .0);
 
- child = gtk_plot_canvas_text_new("Times-Roman", 12, 0, NULL, NULL, TRUE,
+ child = gtk_plot_canvas_text_new("Times-Roman", 12, 270, NULL, NULL, TRUE,
                           GTK_JUSTIFY_CENTER, 
                           "Format text mixing \\Bbold \\N\\i, italics, \\ggreek \\4\\N and \\+different fonts");
  gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .40, .765, .0, .0); 
@@ -487,18 +470,15 @@ int main(int argc, char *argv[]){
  gtk_plot_text_set_border(&GTK_PLOT_CANVAS_TEXT(child)->text, 
                           GTK_PLOT_BORDER_SHADOW, 2, 0, 2);
 
- gtk_plot_text_set_border(&GTK_PLOT(active_plot)->left->title, 
-                          GTK_PLOT_BORDER_SHADOW, 2, 0, 2);
-
  array = GTK_PLOT_ARRAY(gtk_plot_array_new(NULL, custom_labels, 12, GTK_TYPE_STRING, FALSE)); 
  gtk_plot_axis_set_tick_labels(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM), array); 
  gtk_plot_axis_use_custom_tick_labels(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM), TRUE);
 
-// put_child(GTK_PLOT_CANVAS(canvas), .5, .5);
- gtk_widget_show(window1);
+ put_child(GTK_PLOT_CANVAS(canvas), .5, .5);
+
+ gtk_widget_show_all(window1);
 
  gtk_plot_canvas_export_ps(GTK_PLOT_CANVAS(canvas), "demoplot.ps", GTK_PLOT_PORTRAIT, FALSE, GTK_PLOT_LETTER);
-
 /* 
  gtk_plot_canvas_export_ps_with_size(GTK_PLOT_CANVAS(canvas), "demoplot.ps", GTK_PLOT_PORTRAIT, TRUE, GTK_PLOT_PSPOINTS, 300, 400);
 */
@@ -519,9 +499,10 @@ put_child(GtkPlotCanvas *canvas, gdouble x, gdouble y)
 
   pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                                  plot_icon2);
-
   child = gtk_plot_canvas_pixmap_new(pixmap, mask);
+
   gtk_plot_canvas_put_child(canvas, child, x, y, x+.05, y+.05); 
+
   gdk_pixmap_unref(pixmap);
   gdk_bitmap_unref(mask);
 }
