@@ -60,18 +60,15 @@ activate_plot(GtkWidget *widget, gpointer data)
 
   while(n < nlayers)
     {
+      gtk_signal_handler_block_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
       if(widget_list[n] == active_widget){
-            active_plot = plots[n];  
-            GTK_BUTTON(buttons[n])->button_down = TRUE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
+            active_plot = plots[n];
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), TRUE);
       }else{
-            GTK_BUTTON(buttons[n])->button_down = FALSE;
-            GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
-            gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(buttons[n]), FALSE);
       }
-      gtk_widget_queue_draw(buttons[n]);
-      
+      gtk_signal_handler_unblock_by_func(GTK_OBJECT(buttons[n]), GTK_SIGNAL_FUNC(activate_plot), data);
+                                                                                
       n++;
     }
 
@@ -82,6 +79,8 @@ GtkWidget *
 new_layer(GtkWidget *canvas)
 {
  gchar label[10];
+ GtkRequisition req;
+ gint size;
 
  nlayers++;
 
@@ -93,7 +92,9 @@ new_layer(GtkWidget *canvas)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
+ gtk_widget_size_request(buttons[nlayers-1], &req);
+ size = MAX(req.width,req.height);
+ gtk_widget_set_usize(buttons[nlayers-1], size, size);
  gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
